cvs_id = "$Id: Config.py,v 1.5 2003/02/24 10:45:06 juri Exp $"

import cPickle
import os
import traceback

import straw
from error import log

def ensure_directory(strawdir):
    if os.path.exists(strawdir):
        if not os.path.isdir(strawdir):
            raise Exception
        return
    os.mkdir(strawdir)
    return

# using gconf seemed a bit too painful for this, seems it's suitable
# for fixed configuration options but not really much else
class ConfigSaver:
    def __init__(self):
        straw.config.signal_connect(
            straw.ConfigOptionSignal, self.notify_change)
        self._config_file = os.path.join(straw.config._straw_dir, "config")
        self.load_config()
        return

    def notify_change(self, signal):
        self.save_config()
        return

    def load_config(self):
        if os.path.exists(self._config_file):
            fl = straw.FeedList()
            f = open(self._config_file)
            cf = cPickle.load(f)
            for feed in cf['feeds']:
                nf = straw.Feed("")
                nf.undump(feed)
                fl.append(nf)
            straw.config.feeds = fl
            straw.config.poll_frequency = cf.get('poll_frequency', 1200)
            straw.config.number_of_items_stored = cf.get(
                'number_of_items_stored', 30)
            straw.config.item_order = cf.get('item_order', 1)
            straw.config.feed_id_seq = cf.get('feed_id_seq', 0)
            straw.config.main_window_size = cf.get('main_window_size', (700, 550))
            # config.poll_spread 
            f.close()
        return

    def save_config(self):
        cf = {}
        l = []
        for f in straw.config.feeds:
            l.append(f.dump())
        cf['feeds'] = l
        cf['poll_frequency'] = straw.config.poll_frequency
        cf['number_of_items_stored'] = straw.config.number_of_items_stored
        cf['item_order'] = straw.config.item_order
        cf['feed_id_seq'] = straw.config.feed_id_seq
        cf['main_window_size'] = straw.config.main_window_size
        pickle_file = open(self._config_file, 'w')
        cPickle.dump(cf, pickle_file, 1)
        pickle_file.close()
        return

class Config(object, straw.SignalEmitter):
    def __init__(self):
        straw.SignalEmitter.__init__(self)
        self.initialize_slots(straw.FeedsChangedSignal,
                              straw.PollFrequencyChangedSignal,
                              straw.NumberOfItemsStoredChangedSignal,
                              straw.ItemOrderChangedSignal)
        self._poll_frequency = 1200
        self._poll_spread = 0
        self._feed_id_seq = 0
        self._feeds = straw.FeedList()
        self._feeds.signal_connect(straw.FeedsChangedSignal,
                                   self.feeds_changed)
        self._number_of_items_stored = 30
        self._item_order = 1
        self._straw_dir = os.path.join(os.getenv("HOME"), ".straw")
        self._main_window_size = (700, 550)
        ensure_directory(self._straw_dir)
        return

    def feeds_changed(self, signal):
        self.emit_signal(straw.FeedsChangedSignal(self))
    
    def set_feeds(self, feeds):
        self._feeds = straw.FeedList(feeds)
        self._feeds.signal_connect(straw.FeedsChangedSignal,
                                   self.feeds_changed)
        self.emit_signal(straw.FeedsChangedSignal(self))
        return

    def get_feeds(self):
        return self._feeds

    feeds = property(get_feeds, set_feeds, None, "Feeds")

    def set_poll_frequency(self, poll_frequency):
        self._poll_frequency = poll_frequency
        self.emit_signal(straw.PollFrequencyChangedSignal(self))
        return

    def get_poll_frequency(self):
        return self._poll_frequency

    poll_frequency = property(get_poll_frequency, set_poll_frequency, None,
                              "Polling frequency")

    def get_number_of_items_stored(self):
        return self._number_of_items_stored

    def set_number_of_items_stored(self, num):
        self._number_of_items_stored = num
        self.emit_signal(straw.NumberOfItemsStoredChangedSignal(self))

    number_of_items_stored = property(get_number_of_items_stored,
                                      set_number_of_items_stored, None,
                                      "Number of items to store per feed")

    def get_item_order(self):
        return self._item_order

    def set_item_order(self, order):
        self._item_order = order
        self.emit_signal(straw.ItemOrderChangedSignal(self))

    item_order = property(get_item_order, set_item_order, "Ordering of items")

    def get_feed_id_seq(self):
        self._feed_id_seq += 1
        return self._feed_id_seq

    def set_feed_id_seq(self, id):
        self._feed_id_seq = id
        return

    feed_id_seq = property(get_feed_id_seq, set_feed_id_seq, None, "")

    def set_main_window_size(self, size):
        self._main_window_size = size
        self.emit_signal(straw.MainWindowSizeChangedSignal(self))

    def get_main_window_size(self):
        return self._main_window_size

    main_window_size = property(get_main_window_size, set_main_window_size, "")
